part of webui_tasks;

Task createDeployTask(String compiledEntryPoint, String configPath, {String sourcePath:'web', String deployPath:'app', bool debug:false, bool overwrite:true}) {
  final entryPointFile = new File(compiledEntryPoint);
  final configFile = new File(configPath);
  final sourceDir = new Directory(sourcePath);
  final deployDir = new Directory(deployPath);
  
  assert(entryPointFile.existsSync());
  assert(configFile.existsSync());
  assert(sourceDir.existsSync());
  
  JsonObject config = new JsonObject.fromJsonString(configFile.readAsStringSync());
  
  return new Task.async((TaskContext context){
      _deploy_assets(context, config, sourceDir, deployDir, overwrite).whenComplete((){
        
      });
      _fix_url(context, config, entryPointFile, deployDir).whenComplete((){
        
      });
      _deploy_app(context, config, entryPointFile, deployDir, debug).whenComplete((){
        
      });
    return true;
  });
}

Future<bool> _deploy_assets(TaskContext context, JsonObject config, Directory sourceDir, Directory deployDir, bool overwrite){
  var completer = new Completer();
  deployDir.createSync(recursive:true);
  if(overwrite){
    deployDir.deleteSync(recursive:true);
    deployDir.createSync(recursive:true);
  }
  
  var copyFile = (key,value){
    var sourceFile = new File("${sourceDir.path}/$key");
    var targetFile = value =='.' ? new File("${deployDir.path}/$key") : new File("${deployDir.path}/$value");
    context.info("copy from: ${sourceFile.path} to ${targetFile.path}");
    assert(sourceFile.existsSync());
    String source = sourceFile.readAsStringSync();
    context.fine("targetPath: ${targetFile.path}");
    var targetPath = new Path(targetFile.path);
    context.fine("targetDir: ${targetPath.directoryPath}");
    var targetDir = new Directory.fromPath(targetPath.directoryPath);
    context.fine("targetDir exists?: ${targetDir.existsSync()}");
    if(!targetDir.existsSync()) targetDir.createSync(recursive: true);
    targetFile.writeAsStringSync(source);
  };
  
  config.link.forEach(copyFile);
  config.script.forEach(copyFile);
  
  
  return deployDir.exists();
}

Future<bool> _fix_url(TaskContext context, JsonObject config, File compiledEntryPointFile, Directory deployDir){

  context.fine(json.stringify(config.link));
  var document = parse(compiledEntryPointFile.readAsStringSync());
  document
  .queryAll('link')
  .forEach((Element element){
    JsonObject e = new JsonObject.fromMap(element.attributes);
    context.fine("${element.toString()}: ${e.href}");
    if(config.link.containsKey(e.href)){
      String target = config.link[e.href];
      context.fine("config target: $target");
      if(target=='.') target=e.href;
      e.href = target;
    }
  });
  
  context.fine(json.stringify(config.script));
  context.fine(json.stringify(config.app));
  document
  .queryAll('script')
  .forEach((Element element){
    JsonObject e = new JsonObject.fromMap(element.attributes);
    context.fine("${element.toString()}: ${e.src}");
    if(config.script.containsKey(e.src)){
      String target = config.script[e.src];
      context.fine("config target: $target");
      if(target=='.') target=e.src;
      e.src = target;
    }
    if(config.app.containsKey(e.src)){
      String target = config.app[e.src];
      context.fine("config target: $target");
      e.src = target;
    }
  });
  var compiledEntryPath = new Path(compiledEntryPointFile.path);
  var entryPointFileName = compiledEntryPath.filename;
  var deployPath = "${deployDir.path}/$entryPointFileName";
  var deployEntryPoint = new File(deployPath);
  context.info("Deploy compiled entrypoint: ${compiledEntryPath.toString()} to ${deployEntryPoint.toString()}");
  
  deployEntryPoint.writeAsStringSync(document.innerHtml);
  
  return deployEntryPoint.exists();
}

Future<bool> _deploy_app(TaskContext context, JsonObject config, File compiledEntryPointFile, Directory deployDir, bool debug){
  
  String app = config.app.values.first;
  String sourcePath = new Path(compiledEntryPointFile.path).directoryPath;
  context.info("Deploy $app from ${sourcePath} to ${deployDir.path}");
  var sourceFile = new File("$sourcePath/$app");
  var targetFile = new File("${deployDir.path}/$app");
  context.info("Copy from ${sourceFile.toString()} to ${targetFile.toString()}");
  String contents = sourceFile.readAsStringSync();
  targetFile.writeAsStringSync(contents);
  var exists = targetFile.exists();
  context.info("debug: ${debug.toString()}");
  if(debug) {
    sourceFile = new File("$sourcePath/$app.deps");
    targetFile = new File("${deployDir.path}/$app.deps");
    context.info("Copy from ${sourceFile.toString()} to ${targetFile.toString()}");
    String contents = sourceFile.readAsStringSync();
    targetFile.writeAsStringSync(contents);
  }
  
  return exists;
}